clear all
close all
%Output regulation via NESO of a plotter system
%% System specifications
Rm = 1;
Lm = 10e-03;
J1 = 0.01213;
J2 = 0.01213;
B1 = 0.1;
B2 = 0.1;
M = 0.5;
Ki = 1e-03;
Kv = 0.45;
K1 = 1e-03;
K2 = 1e-03;
r1 = 0.1;
r2 = 0.1;
A = [-Rm/Lm 0 -Kv/Lm;
    0 0 1;
    Ki/(J1+(r1/r2)^2*(J2+0.5*M*r2^2)) -(K1+(r1/r2)^2*(K2))/(J1+(r1/r2)^2*(J2+0.5*M*r2^2)) -((B1+(r1/r2)^2*(B2)))/(J1+(r1/r2)^2*(J2+0.5*M*r2^2))];
B = [1/Lm 0 0]';
C = [0 1 0];
%% Exosystem 
wd = 20; %frequency of the disturbance signal
wr = 1; % frequency of the reference signal
S = blkdiag([0 1;-wr^2 0],[0 1;-wd^2 0]);
Cm = [1 0 0 0];
C_sig = [1 0 1 0;0 0 0 0;1 0 1 0];
%% Augmented model
O34 = zeros(3,4);
Lambda = [0 1 0 0;
          0 0 1 0;
          0 0 0 1;
          -wr^2*wd^2 0 -(wr^2+wd^2) 0];
E = [0 0 0 1]';
G = [1 0 0 0];
A_aug = [A O34;
         E*C Lambda];
B_aug = [B;0;0;0;0];
C_aug = [zeros(1,3) G];
%% Nominal regulator gains
K_aug=lqr(A_aug+1.4*eye(size(A_aug)),B_aug,0.001*(C_aug'*C_aug),100);
L_aug=lqr(A_aug'+3.0*eye(size(A_aug)),C_aug',1e-06*eye(7),0.1)';
F1=A_aug-B_aug*K_aug-L_aug*C_aug;
F2=L_aug;
F3=-K_aug;
F4=0;
%% NESO design
epsilon=1;
d=0.1;
sigma=0.8;
S1=S;
C_sig1=C_sig;
A0=[A C_sig1 B;zeros(4,3) S1 zeros(4,1);zeros(1,3) zeros(1,4) 0];
C0=[C zeros(1,4) zeros(1,1)];
beta0=0.01;
%
cvx_begin sdp
variable P(8,8) symmetric
variables Y(8,1) tau(1,1)
P>=(1e-011)*eye(8);
tau>=1e-011;
[P*A0+A0'*P+beta0*P-tau*sigma*(C0'*C0) Y+tau*0.5*(1+sigma)*C0';
    (Y+tau*0.5*(1+sigma)*C0')' -tau]<=0;
cvx_end
H=P\Y;
H1=-H(1:3,:);
H2=-H(4:7,:);
H3=-H(8,:);
%% Simulating the control system
sim('Plotter_system_control');
load('Hybrid_control_system.mat')
figure(1)
set(gcf,'color','w')
plot(t,r,'--r')
hold on
plot(t,y)
hold on
plot(t_hyb,X_hyb(:,1),'g')
legend('Reference','Proposed controller','Hybrid controler')
xlabel('Time (s)','interpreter','latex','fontsize',12)
ylabel('$y,y_m$ (rad)','interpreter','latex','fontsize',12)
%
figure(2)
set(gcf,'color','w')
plot(t,y-r)
hold on
plot(t_hyb,X_hyb(:,1)-xd_hyb(:,1),'g')
legend('Proposed controller','Hybrid controler')
xlabel('Time (s)','interpreter','latex','fontsize',12)
ylabel('$e$ (rad)','interpreter','latex','fontsize',12)
%
figure(3)
set(gcf,'color','w')
plot(t,u_n+u_d)
hold on
plot(t_hyb,u_hyb,'g')
legend('Proposed controller','Hybrid controler')
xlabel('Time (s)','interpreter','latex','fontsize',12)
ylabel('$u$ (V)','interpreter','latex','fontsize',12)
%
figure(4)
set(gcf,'color','w')
subplot(3,1,1)
plot(t,x(:,2))
hold on
plot(t,x_hat(:,2),'r--')
xlim([0,5])
legend('Estimated','True')
ylabel('$x_1,\hat{x}_1$ (rad)','interpreter','latex','fontsize',12)
subplot(3,1,2)
plot(t,x(:,3))
hold on
plot(t,x_hat(:,3),'r--')
xlim([0,5])
ylabel('$x_2,\hat{x}_2$ (rad/s)','interpreter','latex','fontsize',12)
subplot(3,1,3)
plot(t,x(:,1))
hold on
plot(t,x_hat(:,1),'r--')
xlim([0,5])
xlabel('Time (s)','interpreter','latex','fontsize',12)
ylabel('$x_3,\hat{x}_3$ (A)','interpreter','latex','fontsize',12)